@@ -1,12 +1,11 @@
 /* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set shiftwidth=2 tabstop=2 autoindent cindent expandtab: */
 /* globals PDFJS, expect, it, describe, Promise, combineUrl, waitsFor,
-           isArray, MissingPDFException */
+           MissingPDFException, StreamType, FontType */
 
 'use strict';
 
 describe('api', function() {
-  // TODO run with worker enabled
   var basicApiUrl = combineUrl(window.location.href, '../pdfs/basicapi.pdf');
   var basicApiFileLength = 105779; // bytes
   function waitsForPromiseResolved(promise, successCallback) {
@@ -130,6 +129,12 @@ describe('api', function() {
                               0, 841.89, null]);
       });
     });
+    it('gets a non-existent destination', function() {
+      var promise = doc.getDestination('non-existent-named-destination');
+      waitsForPromiseResolved(promise, function(data) {
+        expect(data).toEqual(null);
+      });
+    });
     it('gets attachments', function() {
       var promise = doc.getAttachments();
       waitsForPromiseResolved(promise, function (data) {
@@ -142,6 +147,32 @@ describe('api', function() {
         expect(data).toEqual([]);
       });
     });
+    // Keep this in sync with the pattern in viewer.js. The pattern is used to
+    // detect whether or not to automatically start printing.
+    var viewerPrintRegExp = /\bprint\s*\(/;
+    it('gets javascript with printing instructions (Print action)', function() {
+      // PDF document with "Print" Named action in OpenAction
+      var pdfUrl = combineUrl(window.location.href, '../pdfs/bug1001080.pdf');
+      var promise = PDFJS.getDocument(pdfUrl).then(function(doc) {
+        return doc.getJavaScript();
+      });
+      waitsForPromiseResolved(promise, function (data) {
+        expect(data).toEqual(['print({});']);
+        expect(data[0]).toMatch(viewerPrintRegExp);
+      });
+    });
+    it('gets javascript with printing instructions (JS action)', function() {
+      // PDF document with "JavaScript" action in OpenAction
+      var pdfUrl = combineUrl(window.location.href, '../pdfs/issue6106.pdf');
+      var promise = PDFJS.getDocument(pdfUrl).then(function(doc) {
+        return doc.getJavaScript();
+      });
+      waitsForPromiseResolved(promise, function (data) {
+        expect(data).toEqual(
+          ['this.print({bUI:true,bSilent:false,bShrinkToFit:true});']);
+        expect(data[0]).toMatch(viewerPrintRegExp);
+      });
+    });
     it('gets outline', function() {
       var promise = doc.getOutline();
       waitsForPromiseResolved(promise, function(outline) {
@@ -157,6 +188,7 @@ describe('api', function() {
       var promise = doc.getMetadata();
       waitsForPromiseResolved(promise, function(metadata) {
         expect(metadata.info['Title']).toEqual('Basic API Test');
+        expect(metadata.info['PDFFormatVersion']).toEqual('1.7');
         expect(metadata.metadata.get('dc:title')).toEqual('Basic API Test');
       });
     });
@@ -167,17 +199,16 @@ describe('api', function() {
         expect(data.length).toEqual(basicApiFileLength);
       });
     });
-    it('gets filesize in bytes', function() {
+    it('gets download info', function() {
       var promise = doc.getDownloadInfo();
       waitsForPromiseResolved(promise, function (data) {
-        expect(data.length).toEqual(basicApiFileLength);
+        expect(data).toEqual({ length: basicApiFileLength });
       });
     });
     it('gets stats', function() {
       var promise = doc.getStats();
       waitsForPromiseResolved(promise, function (stats) {
-        expect(isArray(stats.streamTypes)).toEqual(true);
-        expect(isArray(stats.fontTypes)).toEqual(true);
+        expect(stats).toEqual({ streamTypes: [], fontTypes: [] });
       });
     });
   });
@@ -186,10 +217,12 @@ describe('api', function() {
     var promise = new Promise(function (resolve) {
       resolvePromise = resolve;
     });
+    var pdfDocument;
     PDFJS.getDocument(basicApiUrl).then(function(doc) {
       doc.getPage(1).then(function(data) {
         resolvePromise(data);
       });
+      pdfDocument = doc;
     });
     var page;
     waitsForPromiseResolved(promise, function(data) {
@@ -238,5 +271,20 @@ describe('api', function() {
         expect(oplist.lastChunk).toEqual(true);
       });
     });
+    it('gets stats after parsing page', function () {
+      var promise = page.getOperatorList().then(function () {
+        return pdfDocument.getStats();
+      });
+      var expectedStreamTypes = [];
+      expectedStreamTypes[StreamType.FLATE] = true;
+      var expectedFontTypes = [];
+      expectedFontTypes[FontType.TYPE1] = true;
+      expectedFontTypes[FontType.CIDFONTTYPE2] = true;
+
+      waitsForPromiseResolved(promise, function (stats) {
+        expect(stats).toEqual({ streamTypes: expectedStreamTypes,
+                                fontTypes: expectedFontTypes });
+      });
+    });
   });
 });
