@@ -15,7 +15,9 @@
  * limitations under the License.
  */
 /*jshint globalstrict: false */
-/* globals PDFJS, PDFViewer */
+/* globals PDFJS, PDFViewer, PDFPageView, TextLayerBuilder, PDFLinkService,
+           DefaultTextLayerFactory, AnnotationsLayerBuilder, PDFHistory,
+           DefaultAnnotationsLayerFactory, getFileName, ProgressBar */
 
 // Initializing PDFJS global object (if still undefined)
 if (typeof PDFJS === 'undefined') {
@@ -26,7 +28,19 @@ if (typeof PDFJS === 'undefined') {
   'use strict';
 
 //#include ui_utils.js
+//#include pdf_link_service.js
 //#include pdf_viewer.js
+//#include pdf_history.js
 
   PDFJS.PDFViewer = PDFViewer;
+  PDFJS.PDFPageView = PDFPageView;
+  PDFJS.PDFLinkService = PDFLinkService;
+  PDFJS.TextLayerBuilder = TextLayerBuilder;
+  PDFJS.DefaultTextLayerFactory = DefaultTextLayerFactory;
+  PDFJS.AnnotationsLayerBuilder = AnnotationsLayerBuilder;
+  PDFJS.DefaultAnnotationsLayerFactory = DefaultAnnotationsLayerFactory;
+  PDFJS.PDFHistory = PDFHistory;
+
+  PDFJS.getFileName = getFileName;
+  PDFJS.ProgressBar = ProgressBar;
 }).call((typeof window === 'undefined') ? this : window);
